@@ -3,17 +3,16 @@ commands='''
 mkdir fem_tutorial
 cd fem_tutorial
 
-mkdir tmp
-cd tmp
-git init
-git remote add origin https://gitlab.dune-project.org/dune-fem/dune-fempy.git
-git config core.sparsecheckout true
-echo 'demos/' > .git/info/sparse-checkout
-echo 'doc/'  >> .git/info/sparse-checkout
-git pull origin v2.8.0.4
+TMPNAME=`mktemp -d ./tmptutorial.XXXXXX`
+
+# clone repo without history
+git clone --quiet --depth 1 https://gitlab.dune-project.org/dune-fem/dune-fempy.git $TMPNAME
+cd $TMPNAME
+
 cp doc/*.py doc/*.ipynb doc/*.hh doc/*.dgf doc/*.msh ..
-cd ..
-rm -rf tmp
+cd ../
+
+rm -rf $TMPNAME
 rm pandoc-formatting.py gitlab-formatting.py interpolation.py svg2pdf.py
 '''
 subprocess.check_output(commands, shell=True)
