@@ -12,25 +12,25 @@ except ImportError:
 
 from dune.generator.generator import SimpleGenerator
 
-generator = SimpleGenerator("GridAdaptation", "Dune::FemPy")
+_defaultGenerator = SimpleGenerator("GridAdaptation", "Dune::FemPy")
 
 modules = {}
 
 import logging, traceback
 logger = logging.getLogger(__name__)
 
-def module(grid):
+def module(grid, generator=_defaultGenerator):
     try:
-        return modules[grid._typeName]
+        return modules[grid.cppTypeName]
     except KeyError:
         pass
 
-    typeName = "Dune::FemPy::GridAdaptation< " + grid._typeName + " >"
-    includes = grid._includes + ["dune/fempy/py/grid/adaptation.hh"]
+    typeName = "Dune::FemPy::GridAdaptation< " + grid.cppTypeName + " >"
+    includes = grid.cppIncludes + ["dune/fempy/py/grid/adaptation.hh"]
     moduleName = "adapt_" + hashlib.md5(typeName.encode('utf8')).hexdigest()
 
     module = generator.load(includes, typeName, moduleName)
-    modules[grid._typeName] = module
+    modules[grid.cppTypeName] = module
     return module
 
 
@@ -48,6 +48,7 @@ def _adaptArguments(first,*args):
         args = first
     else: # okay apparently its a hgrid object (should be deprecated if args is not empty)
         hgrid = first
+        assert hasattr(hgrid,"leafView"), "second argument is neither a discrete function nor a hierarchical grid"
         if len(args)==0:
             return hgrid,()
         if len(args)==1:
@@ -79,8 +80,10 @@ def loadBalance(first, *args):
     hgrid,args = _adaptArguments(first,*args)
     module(hgrid).gridAdaptation(hgrid).loadBalance(args)
 
-def mark(indicator, refineTolerance, coarsenTolerance=0,
-         minLevel=0, maxLevel=None, gridView=None):
+def mark(indicator, refineTolerance, coarsenTolerance=0.,
+         minLevel=0, maxLevel=None, minVolume=-1.0, maxVolume=-1.0,
+         gridView=None,
+         markNeighbors=False):
     if ufl and (isinstance(indicator, list) or isinstance(indicator, tuple)):
         indicator = ufl.as_vector(indicator)
     if ufl and isinstance(indicator, ufl.core.expr.Expr):#
@@ -93,41 +96,29 @@ def mark(indicator, refineTolerance, coarsenTolerance=0,
             gridView = gridView[0]
         indicator = expression2GF(gridView,indicator,0)
     if gridView is None:
-        # API GridView: gridView = indicator.gridView
-        gridView = indicator.grid
+        gridView = indicator.gridView
     try:
         if not gridView.canAdapt:
             raise AttributeError("indicator function must be over grid view that supports adaptation")
     except AttributeError:
         raise AttributeError("indicator function must be over grid view that supports adaptation")
-    if maxLevel==None:
-        maxLevel = -1
-    return gridView.mark(indicator,refineTolerance,coarsenTolerance,minLevel,maxLevel)
 
-def markNeighbors(indicator, refineTolerance, coarsenTolerance=0,
-                  minLevel=0, maxLevel=None, gridView=None):
-    if ufl and (isinstance(indicator, list) or isinstance(indicator, tuple)):
-        indicator = ufl.as_vector(indicator)
-    if ufl and isinstance(indicator, ufl.core.expr.Expr):#
-        if gridView is None:
-            _, coeff_ = extract_arguments_and_coefficients(indicator)
-            gridView = [c.gridView for c in coeff_ if hasattr(c,"gridView")]
-            # gridView += [c.space.gridView for c in coeff_ if hasattr(c,"space")]
-            if len(gridView) == 0:
-                raise ValueError("if a ufl expression is passed as indicator then the 'gridView' must also be provided")
-            gridView = gridView[0]
-        indicator = expression2GF(gridView,indicator,0)
-    if gridView is None:
-        # API GridView: gridView = indicator.gridView
-        gridView = indicator.grid
-    try:
-        if not gridView.canAdapt:
-            raise AttributeError("indicator function must be over grid view that supports adaptation")
-    except AttributeError:
-        raise AttributeError("indicator function must be over grid view that supports adaptation")
     if maxLevel==None:
         maxLevel = -1
-    return gridView.markNeighbors(indicator,refineTolerance,coarsenTolerance,minLevel,maxLevel)
+    return gridView.mark(indicator,refineTolerance,coarsenTolerance,minLevel,maxLevel, minVolume, maxVolume, markNeighbors)
+
+def markNeighbors(indicator, refineTolerance, coarsenTolerance=0.0,
+                  minLevel=0, maxLevel=None,
+                  minVolume=-1.0, maxVolume=-1.0,
+                  gridView=None):
+    return mark(indicator, refineTolerance,
+                coarsenTolerance=coarsenTolerance,
+                minLevel=minLevel,
+                maxLevel=maxLevel,
+                minVolume=minVolume,
+                maxVolume=maxVolume,
+                gridView=gridView,
+                markNeighbors=True)
 
 def doerflerMark(indicator, theta, maxLevel=None, layered=0.05):
     try:
