@@ -10,7 +10,7 @@ globalBlock = block
 
 def triangulationOfNetwork(grid, level=0, linewidth=0.01):
     from matplotlib.tri import Triangulation
-    x, t = grid.tesselate(level)
+    x, t = grid.tessellate(level)
     n, m = len(x), len(t)
     t = append(t.T, [zeros(m, dtype=int32)], axis=0).T
     t = append(t, zeros((m,3), dtype=int32), axis=0)
@@ -51,7 +51,7 @@ def _plotPointData(fig, grid, solution, level=0, gridLines="black", linewidth=0.
                 data = linalg.norm(solution.pointData(level),axis=1)
             else:
                 data = solution.pointData(level)[:,0]
-            pyplot.plot(grid.tesselate(level)[0], data, '-p')
+            pyplot.plot(grid.tessellate(level)[0], data, '-p')
             if xlim:
                 fig.gca().set_xlim(xlim)
             fig.tight_layout()
@@ -93,6 +93,10 @@ def _plotPointData(fig, grid, solution, level=0, gridLines="black", linewidth=0.
                 logNorm = {}
             minData = amin(data)
             maxData = amax(data)
+            r = maxData-minData
+            # avoid some weird 'white' patches when value hits min/max
+            minData -= r*0.01
+            maxData += r*0.01
             if clim == None:
                 clim = [minData, maxData]
             # having extend not 'both' does not seem to work (needs fixing)...
@@ -151,8 +155,7 @@ def plotPointData(solution, figure=None, linewidth=0.1,
         logscale=False, ticks=11):
     if disable: return
     try:
-        #API GridView: grid = solution.gridView
-        grid = solution.grid
+        grid = solution.gridView
     except AttributeError:
         if isinstance(solution, list) or isinstance(solution,tuple):
             solution = as_vector(solution)
@@ -196,8 +199,7 @@ def plotComponents(solution, figure=None, level=0, show=None, gridLines="black",
         block=globalBlock, grid=None, colorbar=None):
     if disable: return
     try:
-        # API GridView: grid = solution.gridView
-        grid = solution.grid
+        grid = solution.gridView
     except AttributeError:
         if isinstance(solution, Expr):
             assert grid, "need to provide a named grid argument to plot a ufl expression directly"
