@@ -14,7 +14,6 @@ from dune.source.cplusplus import Include, Method, UnformattedExpression,\
      UnformattedBlock, Variable, Struct, TypeAlias, Constructor, return_, IfStatement
 from dune.source.cplusplus import assign
 from dune.source.cplusplus import ListWriter, StringWriter, SourceWriter
-from dune.source import BaseModel
 from dune.source.fem import declareFunctionSpace
 import ufl
 from ufl import Coefficient, as_vector, replace
@@ -30,13 +29,13 @@ from dune.ufl.codegen import uflSignature, TooHighDerivative
 from dune.ufl import codegen
 
 class UFLFunctionSource(codegen.ModelClass):
-    version = "v1_2"
+    version = "v1_3"
     def __init__(self, grid, expr,
             name,order,
             tempVars=True, virtualize=True,
             predefined=None):
-        gridType = grid._typeName
-        gridIncludes = grid._includes
+        gridType = grid.cppTypeName
+        gridIncludes = grid.cppIncludes
         if len(expr.ufl_shape) == 0:
             expr = as_vector( [ expr ] )
         dimRange = expr.ufl_shape[0]
@@ -148,7 +147,7 @@ class UFLFunctionSource(codegen.ModelClass):
                 code.append(Include("dune/fempy/function/virtualizedgridfunction.hh"))
             else:
                 for c in self.coefficientList:
-                    for i in c._includes:
+                    for i in c.cppIncludes:
                         code.append(Include(i))
         code.append(Include("dune/fempy/py/ufllocalfunction.hh"))
 
