@@ -69,9 +69,15 @@ class CodeGenerator(MultiFunction):
         self.using.add(Using(cplusplus.abs_))
         return self._makeTmp(cplusplus.abs_(x))
 
-    def AndCondition(self, expr, left, right):
+    def not_condition(self, o, x):
+        self.using.add(Using(cplusplus.not_))
+        return self._makeTmp(cplusplus.not_(x))
+    def and_condition(self, expr, left, right):
         self.using.add(Using(cplusplus.and_))
         return self._makeTmp(cplusplus.and_(left, right))
+    def or_condition(self, expr, left, right):
+        self.using.add(Using(cplusplus.or_))
+        return self._makeTmp(cplusplus.or_(left, right))
 
     def argument(self, expr):
         try:
@@ -87,6 +93,8 @@ class CodeGenerator(MultiFunction):
         self.using.add(Using(cplusplus.atan2))
         return self._makeTmp(cplusplus.atan2(x, y))
 
+    boundary_id = _require_predefined
+
     cell_volume = _require_predefined
 
     def coefficient(self, expr):
@@ -346,12 +354,12 @@ def fieldVectorType(shape, field = None, useScalar = False):
 
 def gridPartType(gf):
     try:
-        gv = gf.space.gridView._typeName
+        gv = gf.space.gridView.cppTypeName
     except AttributeError:
         try:
-            gv = gf.gridView._typeName
+            gv = gf.gridView.cppTypeName
         except AttributeError:
-            gv = gf.grid._typeName
+            gv = gf.grid.cppTypeName
     gvType = re.split('::GridViewType$', gv)
     if len(gvType) == 2: # is a dune fem grid part
         return gvType[0]
@@ -459,16 +467,16 @@ class ModelClass():
                 self.coefficientCppTypes = \
                     ['Dune::FemPy::VirtualizedGridFunction< ' +\
                      gridPartType(c) + ', ' + fieldVectorType(c) + ' >' \
-                        if not c._typeName.startswith("Dune::Python::SimpleGridFunction") \
-                        else c._typeName \
+                        if not c.cppTypeName.startswith("Dune::Python::SimpleGridFunction") \
+                        else c.cppTypeName \
                     for c in self.coefficientList]
                 # VirtualizedGF need GridParts but they have to be the same for all coefficients
                 # Do we want this to work in some way? Then we can add
                 # includes here - but '.bind(entity)' will fail.
                 # for c in self.coefficientList:
-                #     self.includeFiles += c.grid._includes
+                #     self.includeFiles += c.grid.cppIncludes
             else:
-                self.coefficientCppTypes = [c._typeName for c in self.coefficientList]
+                self.coefficientCppTypes = [c.cppTypeName for c in self.coefficientList]
         else:
             self.coefficientCppTypes = []
 
@@ -552,6 +560,9 @@ class ModelClass():
         entity = 'entity()' if side is None else 'entity_[ static_cast< std::size_t >( ' + side + ' ) ]'
         return UnformattedExpression('auto', entity + '.geometry().volume()')
 
+    def intersection(self):
+        return UnformattedExpression('auto', 'intersection_')
+
     def cellGeometry(self, side=None):
         entity = 'entity()' if side is None else 'entity_[ static_cast< std::size_t >( ' + side + ' ) ]'
         return UnformattedExpression('auto', entity + '.geometry()')
